/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx;

/**
 * Common values for PD game
 * 
 * @author Tomasz Kaczanowski, Jan Humble
 * @version 1.0.1
 */
public interface PDValues {
	/**
	 * names of available moves - cooperation, defection and exit the third move -
	 * exit - will be used in games during which players can decide when to stop
	 * playing
	 */
	final static String[] moves = { "COOPERATION", "DEFECTION", "EXIT" };

	// constant values for moves' names

	/** constant value for COOPERATE move */
	final static double COOPERATE = 1.0;

	/** constant value for DEFECT move */
	final static double DEFECT = 0.0;

	/**
	 * constant value for EXIT move not used yet, but if you want to give your
	 * strategies an 'exit' option you can use this value to indicate this
	 */
	final static double EXIT = 2;

	// constant values for players

	/** constant value for PLAYER A */
	final static int PLAYER_A = 0;

	/** constant value for PLAYER B */
	final static int PLAYER_B = 1;

	// constant values for default game matrix values

	/** default value for sucker's payoff */
	final static double DEFAULT_SUCKER = 0.0;

	/** default value for punishment */
	final static double DEFAULT_PUNISHMENT = 2.0;

	/** default value for reward */
	final static double DEFAULT_REWARD = 4.0;

	/** default value for temptation to defect */
	final static double DEFAULT_TEMPTATION = 5.0;

	/** default value for probability of mistake level */
	float DEFAULT_MISTAKE_LEVEL = 0.0f;
}
