/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx;

/**
 * base class for all players
 * 
 * @author Tomasz Kaczanowski, Jan Humble
 */
public class Player {
	/** sum of points won by player */
	private double result;

	/** player's strategy */
	protected Strategy strategy;

	protected Class strategyClass;

	/** player's name */
	protected String name;

	public int gamesPlayed, gamesWon, gamesLost, gamesTied;

	/**
	 * @param strategy
	 */
	public Player(Strategy strategy) {
		this(strategy, "");
	}

	/**
	 * @param strategy
	 * @param name
	 */
	public Player(Strategy strategy, String name) {
		setStrategy(strategy);
		this.name = name;
	}

	/**
	 * Returns the player name.
	 */
	public String getName() {
		return name;
	}

	public final void setStrategy(Strategy strategy) {
		this.strategy = strategy;
		if (strategy != null) {
			this.strategyClass = strategy.getClass();
		}
	}

	/**
	 * increases result by a given number also increases player's strategy
	 * result by the same number
	 * 
	 * @param number
	 *            number of points to be added to result
	 */
	void increaseResult(double number) {
		result += number;
	}

	/**
	 * returns result
	 * 
	 * @return number of points gathered by this player during game(s)
	 */
	public double getResult() {
		return result;
	}

	public void clearResult() {
		this.result = 0.0;
		gamesPlayed = 0;
		gamesWon = 0;
		gamesLost = 0;
		gamesTied = 0;
	}

	/**
	 * clears the memory of the player - sets his strategy to the initial state
	 */
	public void clearMemory() {
		strategy.reset();
	}

	/**
	 * returns move
	 * 
	 * @return move of this player
	 */
	public double getMove() {
		return strategy.getMove();
	}

	/**
	 * setter for opponentMove field
	 * 
	 * @param opponentMove
	 *            last move of the opponent
	 */
	public void setOpponentMove(double opponentMove) {
		strategy.opponentMove = opponentMove;
	}

	/**
	 * setter for opponentMove field
	 * 
	 * @param opponentsMoves
	 *            last moves of the opponents
	 */
	public void setOpponentsMoves(double[] opponentsMoves) {
		strategy.opponentsMoves = opponentsMoves;
	}

	public final void setNumberOfOpponents(int nrOfOpponents) {
		strategy.setNumberOfOpponents(nrOfOpponents);
	}

	public void setLastResult(double lastResult) {
		strategy.lastResult = lastResult;
	}

	public Strategy getStrategy() {
		return getStrategy(false);
	}

	public void reloadStrategy() {
		this.strategy = getStrategy(true);
	}

	protected Strategy getStrategy(boolean newInstance) {
		if (newInstance) {
			try {
				return (Strategy) strategy.clone();
			} catch (CloneNotSupportedException cnse) {
				cnse.printStackTrace();
				return null;
			}
		}
		return strategy;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Player: " + super.toString() + "\n\tstrategy - "
				+ strategy.getAbbrName() + "\tpoints - " + getResult();
	}
}
