/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx;

/**
 * payoff matrix for a player. This class has two purposes: 1) helps create game
 * matrices 2) allows to easily print out the matrix for one player
 * 
 * @author Tomek Kaczanowski
 */
public class PlayerMatrix implements PDValues {
	/**
	 * 2x2 matrix of payoffs for one player
	 */
	protected double[][] matrix;

	/**
	 * constructor
	 * 
	 * @param reward -
	 *            reward for cooperation
	 * @param sucker -
	 *            sucker's payoff
	 * @param temptation -
	 *            temptation to defect
	 * @param punishment -
	 *            punishment (for mutual defection)
	 */
	public PlayerMatrix(double reward, double sucker, double temptation,
			double punishment) {
		matrix = new double[][] { { reward, sucker },
				{ temptation, punishment } };
	}

	public PlayerMatrix(double[][] matrix) {
		this.matrix = matrix;
	}

	/**
	 * @return matrix with player's payoffs
	 */
	public double[][] getMatrix() {
		return matrix;
	}

	/**
	 * Prints out player's payoffs
	 * 
	 * @return String with nicely formatted matrix
	 */
	public String toString() {
		StringBuffer out = new StringBuffer();
		for (int j = 0; j < matrix.length; j++) {
			for (int i = 0; i < matrix[0].length; i++) {
				out.append(matrix[j][i] + "\t");
			}
			out.append("\n");
		}

		return out.toString();
	}

	/**
	 * takes PlayerMatrix and changes places of temptation and sucker. usefull
	 * for creating symmetrical game matrices
	 * 
	 * @return PlayerMatrix with changed places of temptation and sucker
	 */
	public PlayerMatrix transpose() {
		double[][] matrix = getMatrix();
		/*
		 * return new PlayerMatrix(matrix[COOPERATE][COOPERATE],
		 * matrix[DEFECT][COOPERATE], matrix[COOPERATE][DEFECT],
		 * matrix[DEFECT][DEFECT]);
		 */
		return new PlayerMatrix(matrix);
	}
}
