/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.examples;

import ipdlx.Game;
import ipdlx.GameListener;
import ipdlx.GameMatrix;
import ipdlx.GameResult;
import ipdlx.GameRoundResult;
import ipdlx.StandardGame;
import ipdlx.Strategy;
import ipdlx.Tournament;
import ipdlx.TournamentListener;
import ipdlx.TournamentResult;
import ipdlx.strategy.*;

import java.util.Vector;

/**
 * 
 * @author Jan Humble
 */
public class CMCTournamentTest implements TournamentListener, GameListener {

	Game game;

	Tournament tournament;

	Vector players;
	
	Strategy testStrategy;

	public static void main(String[] args) {
		String testStrategyName = args[0];
		Strategy s1;
		try {
			s1 = ((Strategy)(Class.forName("ipdlx.strategy." + testStrategyName).newInstance()));
			System.out.println(s1.getName());
		} catch (Exception e) {
			System.out.println("Unable to instantiate class " + testStrategyName);
			return;
		}
		CMCTournamentTest testGame1 = new CMCTournamentTest(s1);
		testGame1.doTheTest();
	}
	
	public CMCTournamentTest(Strategy s1) {
		testStrategy = s1;
	}

	public void gameResultPosted(GameResult result) {
		System.out.println(result);
	}

	public void gameRoundResultPosted(GameRoundResult result) {
		System.out.println(result);
	}

	public void gameStarted(Game game) {
	}

	public void tournamentResultPosted(TournamentResult result) {
		System.out.println(result);
	}

	public void tournamentStarted(Tournament t) {
	}

	public void createPlayers() {
		// create players
		players = new Vector();
		players.addAll(Tournament.createPlayers(new RAND(), 1));
		players.addAll(Tournament.createPlayers(new ALLC(), 1));
		players.addAll(Tournament.createPlayers(new ALLD(), 1));
		players.addAll(Tournament.createPlayers(testStrategy, 1));
	}

	public void createTournament() {
		// first create a game matrix
		GameMatrix gameMatrix = new GameMatrix();

		System.out.println(gameMatrix);

		// set number of iterations in game
		int nbOfRounds = 100;

		// create game
		game = new StandardGame(nbOfRounds, gameMatrix);
		tournament = new Tournament(game);
		createPlayers();
		tournament.addPlayers(players);

	}

	public void doTheTest() {
		createTournament();
		game.addGameListener(this);
		tournament.addTournamentListener(this);
		// let's begin
		tournament.play();
	}
}
