/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.gui;

import ipdlx.Game;
import ipdlx.MultiPlayerGame;
import ipdlx.StandardGame;
import ipdlx.Tournament;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

/**
 *
 * @author Jan Humble
 */
public class GameSettingsPanel extends JPanel implements ActionListener {

    private final Tournament tournament;
    private JFormattedTextField roundsTF, noiseTF, stDevTF;
    private JComboBox typeChoice;
    private JButton applyB;

    private static final String[] GAME_TYPES = 
	new String[] {Tournament.gameTypeToString(Tournament.STANDARD_GAME),
		      Tournament.gameTypeToString(Tournament.MULTIPLAYER_GAME)};
    
    public GameSettingsPanel(Tournament tournament) {
	super(new BorderLayout());
	this.tournament = tournament;
	add(BorderLayout.NORTH, new GameMatrixPanel(tournament));
	
	JPanel controls = new JPanel(new BorderLayout());
	JPanel settingsP = new JPanel(new GridLayout(4,1));
	controls.setBorder(new EmptyBorder(5, 0, 5, 0));
	JPanel p = new JPanel(new GridLayout(1, 2));
	p.add(new JLabel("Rounds per game: "));
	roundsTF = 
	    new JFormattedTextField(new Integer(tournament.getGame().getNumberOfRounds()));
	
	roundsTF.addActionListener(this);
	p.add(roundsTF);
	settingsP.add(p);
	p = new JPanel(new GridLayout(1, 2));
	stDevTF = new JFormattedTextField(new Float(tournament.getGame().getNrRoundsStDev()));
	p.add(new JLabel("St Dev:"));
	p.add(stDevTF);
	stDevTF.addActionListener(this);
	settingsP.add(p);

	p = new JPanel(new GridLayout(1, 2));
	p.add(new JLabel("Noise: "));
	noiseTF = new JFormattedTextField(new Float(tournament.getGame().getProbabilityOfMistake()));
	noiseTF.addActionListener(this);
	p.add(noiseTF);
	settingsP.add(p);

	p = new JPanel(new GridLayout(1, 2));
	p.add(new JLabel("Game Type: "));
	p.add(typeChoice = new JComboBox(GAME_TYPES));
	typeChoice.addActionListener(this);
	settingsP.add(p);

	controls.add(BorderLayout.NORTH, settingsP);
	JPanel applyP = new JPanel();
	applyB = new JButton("Edit");
	applyB.addActionListener(this);
	applyP.add(applyB);
	controls.add(BorderLayout.SOUTH, applyP);
	
	JPanel resultsP = new JPanel(new GridLayout(2, 1));
	JCheckBox outputCB = new JCheckBox("Show game results", 
					   tournament.getGame().getBroadcastResult());
	outputCB.addActionListener(this);
	resultsP.add(outputCB);
	outputCB = new JCheckBox("Show round results", 
				 tournament.getGame().getBroadcastRoundResults());
	outputCB.addActionListener(this);
	resultsP.add(outputCB);
	
	p = new JPanel(new BorderLayout());
	p.add(BorderLayout.NORTH, controls);
	p.add(BorderLayout.SOUTH, resultsP);
	add(BorderLayout.SOUTH, p);
	setEnabled(false);
    }

    public void actionPerformed(ActionEvent ae) {
	int rounds = Integer.parseInt(roundsTF.getText());
	float noise = Float.parseFloat(noiseTF.getText());
	float stDev = Float.parseFloat(stDevTF.getText());
	Object obj = ae.getSource();
	String command = ae.getActionCommand();
	if (command.equals("Show game results")) {
	    tournament.getGame().setBroadcastResult(((JCheckBox) obj).isSelected());
	} else if (command.equals("Show round results")) {
	    tournament.getGame().setBroadcastRoundResults(((JCheckBox) obj).isSelected());
	} else if (command.equals("Apply")) {
	    tournament.getGame().setNumberOfRounds(rounds);
	    tournament.getGame().setProbabilityOfMistake(noise);
	    tournament.getGame().setNrRoundsStDev(stDev);
	    String gameType = (String) typeChoice.getSelectedItem();
	    setGameType(gameType);
	    setEnabled(false);
	    applyB.setText("Edit");
	} else if (command.equals("Edit")) {
	    setEnabled(true);
	    applyB.setText("Apply");
	}
    }
    
    public void setEnabled(boolean enable) {
	roundsTF.setEnabled(enable);
	noiseTF.setEnabled(enable);
	stDevTF.setEnabled(enable);
	typeChoice.setEnabled(enable);
	super.setEnabled(enable);
    }

    private void setGameType(String type) {
	Vector listeners = tournament.getGame().getGameListeners();
	Game newGame = tournament.getGame();
	if (type.equals(GAME_TYPES[0])) {
	    newGame = new StandardGame(newGame);
	} else if (type.equals(GAME_TYPES[1])) {
	    newGame = new MultiPlayerGame(newGame);
	}
	tournament.setGame(newGame);
	System.out.print("Setting new game parameters ...\n" + newGame.toString());
    }
    
}