/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.gui;

import ipdlx.Player;
import ipdlx.Tournament;
import ipdlx.TournamentListener;
import ipdlx.TournamentResult;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Font;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;

/**
 * 
 * @author Jan Humble
 */
public class ResultsPanel extends JPanel implements TournamentListener,
		ActionListener {

	private JLabel timeTF;

	private ResultsTable resultsTable;

	private TournamentResult lastResult;

	private File chosenFile, chosenDirectory;

	public ResultsPanel(Tournament t) {
		super(new BorderLayout());
		t.addTournamentListener(this);
		JPanel top = new JPanel();
		top.add(new JLabel("Total run time:"));
		top.add(timeTF = new JLabel("0.0 secs"));
		add(BorderLayout.NORTH, top);
		add(BorderLayout.CENTER, new JScrollPane(
				resultsTable = new ResultsTable(t)));
		JPanel bottom = new JPanel();
		JButton b = new JButton("Create HTML");
		b.addActionListener(this);
		bottom.add(b);
		add(BorderLayout.SOUTH, bottom);
	}

	public void actionPerformed(ActionEvent ae) {
		if (lastResult != null) {
			SimpleDateFormat dateFormat = new SimpleDateFormat(
					"yyyyMMdd_HHmmss");
			String date = dateFormat.format(lastResult.getTournamentDate());
			chosenFile = chooseSaveFile("pd_tournament_result_" + date
					+ ".html");
			if (chosenFile != null) {
				String html = ipdlx.tools.HTMLGenerator
						.tournamentResultToHTML(lastResult);
				try {
					FileWriter fw = new FileWriter(chosenFile);
					fw.write(html, 0, html.length());
					fw.close();
				} catch (IOException ioe) {
					ioe.printStackTrace();
				}
			}
		}
	}

	private File chooseSaveFile(String defaultName) {
		if (chosenDirectory == null) {
			chosenDirectory = new File(".");
		}
		JFileChooser chooser = new JFileChooser(chosenDirectory);
		HTMLFileFilter filter = new HTMLFileFilter();
		chooser.setFileFilter(filter);
		chooser.setSelectedFile(new File(defaultName));
		int returnVal = chooser.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			System.out.println("You chose to save to this file: "
					+ chooser.getSelectedFile().getName());
			chosenDirectory = chooser.getCurrentDirectory();
			return chooser.getSelectedFile();
		}
		return null;
	}

	public void tournamentResultPosted(TournamentResult result) {
		this.lastResult = result;
		result.sortPlayers();
		timeTF.setText(result.getTotalTournamentTime() / 1000.0 + " secs");
		resultsTable.setFont( new Font("Default", Font.PLAIN, 24));
		resultsTable.setRowHeight(30);
		resultsTable.setPlayers(result.getPlayers());
	}

	public void tournamentStarted(Tournament t) {
	}
}

class HTMLFileFilter extends FileFilter {

	public boolean accept(File f) {
		if (!f.canRead()) {
			return false;
		}

		if (f.isFile()) {
			String filename = f.getName();
			if (filename.endsWith(".html")) {
				return true;
			}
		} else if (f.isDirectory()) {
			return true;
		}
		return false;
	}

	public String getDescription() {
		return "HTML File";
	}
}

class ResultsTable extends JTable {

	public ResultsTable(Tournament t) {
		super(new ResultTableModel(t.getPlayers()));
	}

	public void setPlayers(Vector players) {
		((ResultTableModel) getModel()).setPlayers(players);
		revalidate();
		repaint();
	}

}

class ResultTableModel extends DefaultTableModel {

	private static String[] headers = { "Ranking", "Strategy",
			"Won", "Tied", "Lost", "Total" };

	private Vector players;

	ResultTableModel(Vector players) {
		super(headers, players.size());
		this.players = players;
	}

	public void setPlayers(Vector players) {
		this.players = players;
	}

	public String getColumnName(int column) {
		return headers[column];
	}

	public int getRowCount() {
		if (players != null) {
			return players.size();
		}
		return 0;
	}

	public Object getValueAt(int row, int column) {
		if (players == null) {
			return null;
		}
		Player p = (Player) players.elementAt(row);
		switch (column) {
		case 0:
			return new Integer(row + 1);
		case 1:
			return p.getStrategy().getName();
		case 2:
			return new Integer(p.gamesWon);
		case 3:
			return new Integer(p.gamesTied);
		case 4:
			return new Integer(p.gamesLost);
		case 5:
			return new Double(p.getResult());
		default:
			return "";
		}
	}
}

/*
class ResultTableModel extends DefaultTableModel {

	private static String[] headers = { "Ranking", "Player", "Strategy",
			"Games Played", "Won", "Tied", "Lost", "Total" };

	private Vector players;

	ResultTableModel(Vector players) {
		super(headers, players.size());
		this.players = players;
	}

	public void setPlayers(Vector players) {
		this.players = players;
	}

	public String getColumnName(int column) {
		return headers[column];
	}

	public int getRowCount() {
		if (players != null) {
			return players.size();
		}
		return 0;
	}

	public Object getValueAt(int row, int column) {
		if (players == null) {
			return null;
		}
		Player p = (Player) players.elementAt(row);
		switch (column) {
		case 0:
			return new Integer(row + 1);
		case 1:
			return p.getName();
		case 2:
			return p.getStrategy().getFullName();
		case 3:
			return new Integer(p.gamesPlayed);
		case 4:
			return new Integer(p.gamesWon);
		case 5:
			return new Integer(p.gamesTied);
		case 6:
			return new Integer(p.gamesLost);
		case 7:
			return new Double(p.getResult());
		default:
			return "";
		}
	}
}
*/