/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.gui;

import ipdlx.Strategy;
import ipdlx.Tournament;
import ipdlx.tools.PDFileHandler;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

public class StrategyPanel extends JPanel implements ActionListener {

	private Tournament tournament;

	private JTextField[] nrPlayersTF;

	private Hashtable players;

	private JTextField jarFileTF, playerNameTF;

	private JTextField nrPlayersCreateTF;

	private File chosenFile;
	private File chosenDirectory;

	private Strategy chosenStrategy;

	public final static String[] DEFAULT_STRATEGIES = new String[] {
			"ipdlx.strategy.RAND", "ipdlx.strategy.ALLC",
			"ipdlx.strategy.ALLD" };

	public final static String[] MULTICHOICE_DEFAULT_STRATEGIES = new String[] { "ipdlx.strategy.MRAND" };

	public StrategyPanel(Tournament tournament) {
		this.tournament = tournament;
		this.nrPlayersTF = new JTextField[DEFAULT_STRATEGIES.length];
		BorderedPanel p = new BorderedPanel("Strategies");
		for (int i = 0; i < DEFAULT_STRATEGIES.length; i++) {
			JPanel stratPanel = createStrategyChoicePanel(i,
					DEFAULT_STRATEGIES[i]);
			if (stratPanel != null) {
				p.add(stratPanel);
			}
		}
		add(p);

		players = new Hashtable();
		add(createLoadPanel());
	}

	JPanel createLoadPanel() {
		BorderedPanel p = new BorderedPanel("Create Player");

		JPanel panel = new JPanel(new FlowLayout());
		panel.add(new JLabel("Strategy File:"));
		panel.add(jarFileTF = new JTextField(20));

		JButton browseB = new JButton("Browse");
		browseB.addActionListener(this);
		panel.add(browseB);
		p.add(panel);
		panel = new JPanel();
		panel.add(new JLabel("Player Name:"));
		playerNameTF = new JTextField("Player");
		playerNameTF.setColumns(10);
		panel.add(playerNameTF);
		this.nrPlayersCreateTF = new JFormattedTextField(new Integer(1));
		nrPlayersCreateTF.setColumns(4);

		panel.add(new JLabel("Nr Players:"));
		panel.add(nrPlayersCreateTF);
		JButton submitB = new JButton("Create");
		submitB.addActionListener(this);
		panel.add(submitB);
		p.add(panel);
		return p;
	}

	JPanel createStrategyChoicePanel(final int index, String strategyName) {
		JPanel p = new JPanel(new BorderLayout());
		// Verify if it exists
		final Class strategyClass = Tournament.getStrategyClass(strategyName);
		if (strategyClass == null) {
			return null;
		}
		final JCheckBox cb = new JCheckBox(strategyClass.getName());
		nrPlayersTF[index] = new JFormattedTextField(new Integer(1));
		nrPlayersTF[index].setColumns(4);
		cb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				String command = ae.getActionCommand();
				if (cb.isSelected()) {
					int size = Integer.parseInt(nrPlayersTF[index].getText());
					Vector newPlayers = Tournament.createPlayers(null,
							strategyClass, size);
					players.put(command, newPlayers);
					tournament.addPlayers(newPlayers);
					TournamentPanel.monitor.fireChange();
				} else {
					Vector deleteP = (Vector) players.get(command);
					Vector tPlayers = tournament.getPlayers();
					for (Enumeration e = deleteP.elements(); e
							.hasMoreElements();) {
						tPlayers.remove(e.nextElement());
					}
					players.remove(command);
					TournamentPanel.monitor.fireChange();
				}
			}
		});
		p.add(BorderLayout.WEST, cb);
		JPanel pp = new JPanel();
		pp.add(new JLabel("Nr Players:"));

		pp.add(nrPlayersTF[index]);
		p.add(BorderLayout.EAST, pp);
		return p;
	}

	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		if (command.equals("Browse")) {
			chosenFile = chooseFile();
			if (chosenFile != null) {
				try {
					if (chosenFile.getName().endsWith(".jar")) {
						PDFileHandler fileHandler = new PDFileHandler(
								new URL[] {
										PDFileHandler.fileToJarURL(chosenFile,
												""),
										PDFileHandler.fileToJarURL(chosenFile,
												"classes/") });
						chosenStrategy = fileHandler.getStrategyFromJarFile(
								chosenFile, null);
					} else if (chosenFile.getName().endsWith(".class")) {
						PDFileHandler fileHandler = new PDFileHandler(
								chosenDirectory.toURL());
						System.out.println(chosenDirectory.toURL());
						chosenStrategy = fileHandler.getStrategy(chosenFile
								.getName());
						System.out.println(chosenFile.getName());
						System.out.println(chosenStrategy.getName());
					}
					jarFileTF.setText(chosenStrategy.getName());
				} catch (MalformedURLException mue) {
					mue.printStackTrace();
				}
			} else {
				System.out.println("Error: Chosen file not legal");
			}
		} else if (command.equals("Create")) {
			String name = playerNameTF.getText().trim();
			int nrPlayers = Integer.parseInt(nrPlayersCreateTF.getText());
			Vector players = Tournament.createPlayers(name, chosenStrategy,
					nrPlayers);
			tournament.addPlayers(players);
			TournamentPanel.monitor.fireChange();
		}
	}

	private File chooseFile() {
		// ATF modification
		String dirPath = "/Users/atf/Documents/cmc_class/ipdlx_src/ipdlx/strategy";
		JFileChooser chooser = new JFileChooser(dirPath);
		PDJARFileFilter filter = new PDJARFileFilter();
		chooser.setFileFilter(filter);
		int returnVal = chooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			System.out.println("You chose to open this file: "
					+ chooser.getSelectedFile().getName());

			chosenDirectory = chooser.getCurrentDirectory();
			return chooser.getSelectedFile();
		}
		return null;
	}
}

class PDJARFileFilter extends FileFilter {

	public boolean accept(File f) {

		if (!f.canRead()) {
			return false;
		}

		if (f.isFile()) {
			String filename = f.getName();
			if (filename.endsWith(".jar") || filename.endsWith(".class")) {
				return true;
			}
		} else if (f.isDirectory()) {
			return true;
		}

		return false;
	}

	public String getDescription() {
		return "IPDLX Strategy files (.class, .jar)";
	}
}