/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.gui;

import ipdlx.Game;
import ipdlx.GameListener;
import ipdlx.GameResult;
import ipdlx.GameRoundResult;
import ipdlx.Tournament;
import ipdlx.TournamentListener;
import ipdlx.TournamentResult;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

/**
 * 
 * @author Jan Humble
 */
public class TournamentRunPanel extends JPanel implements ActionListener,
		GameListener, TournamentListener, Observer {

	private Tournament tournament;

	private JProgressBar progressBar;

	private JButton startButton, stopButton, clearButton;

	private int totalGames, gamesRun;

	public TournamentRunPanel(Tournament tournament) {
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.progressBar = new JProgressBar();
		this.progressBar.setStringPainted(true);
		this.progressBar.setMinimum(0);
		this.progressBar.setForeground(java.awt.Color.green);
		this.startButton = new JButton("Start");
		this.startButton.addActionListener(this);
		this.stopButton = new JButton("Stop");
		this.stopButton.addActionListener(this);
		this.clearButton = new JButton("Clear");
		this.clearButton.addActionListener(this);

		JPanel controls = new JPanel();
		controls.add(startButton);
		controls.add(stopButton);
		controls.add(clearButton);
		add(BorderLayout.NORTH, controls);
		add(BorderLayout.SOUTH, progressBar);

		setTournament(tournament);
	}

	public void setTournament(Tournament tournament) {
		this.tournament = tournament;
		TournamentPanel.monitor.addObserver(this);
		this.tournament.addTournamentListener(this);
		this.tournament.getGame().addGameListener(this);
		clear();
	}

	public void update(Observable o, Object arg) {
		setValues();
	}

	void setValues() {
		totalGames = tournament.getNrOfGames();
		progressBar.setMaximum(totalGames);
		progressBar.setString("Games to run: " + totalGames);
	}

	public void clear() {
		setValues();
		gamesRun = 0;
		progressBar.setValue(0);
		stopButton.setEnabled(false);
		startButton.setEnabled(true);
		clearButton.setEnabled(true);
		tournament.clearAllResults();
	}

	public void stop() {
		tournament.stop();
		stopButton.setEnabled(false);
		startButton.setEnabled(true);
		clearButton.setEnabled(true);
	}

	public void start() {
		setValues();
		stopButton.setEnabled(true);
		startButton.setEnabled(false);
		clearButton.setEnabled(false);

		System.out.println("Starting tournament on "
				+ java.util.Calendar.getInstance().getTime() + " ...\n"
				+ tournament.getGame().toString());
		tournament.play();
	}

	public void tournamentResultPosted(TournamentResult result) {
		stop();
	}

	public void tournamentStarted(Tournament t) {
	}

	public void gameStarted(Game game) {
	}

	public void gameResultPosted(GameResult result) {
		progressBar.setValue(++gamesRun);
		progressBar.setString(gamesRun + "/" + totalGames);
	}

	public void gameRoundResultPosted(GameRoundResult result) {
	}

	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		if (command.equals("Start")) {
			start();
		} else if (command.equals("Stop")) {
			stop();
		} else if (command.equals("Clear")) {
			clear();
		}
	}

}