/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.tools;

import ipdlx.Player;
import ipdlx.Tournament;
import ipdlx.TournamentResult;

import java.text.MessageFormat;
import java.util.Enumeration;

public abstract class HTMLGenerator {

	private static final String HEADER = "<HTML>\n"
			+ "<HEAD>\n"
			+ "<TITLE>PD Tournament Result</TITLE>\n"
			+ "</HEAD>\n"
			+ "<BODY TEXT=\"#000000\" BGCOLOR=\"#FFFFFF\" LINK=\"#0F0CD9\" VLINK=\"#E90716\">\n";

	private static final String FOOTER = "\n<HR>\n<BR>Generated by IPDLX libraries<BR>Author: <A HREF=\"mailto:jch@cs.nott.ac.uk\">Jan Humble</A><BR>Contact: <A HREF=\"mailto:gxk@cs.nott.ac.uk\">Graham Kendall</A><BR> </HTML>\n";

	public static String tournamentResultToHTML(TournamentResult result) {
		StringBuffer html = new StringBuffer(HEADER);
		html.append("<H1>Prisoner's Dilemma Tournament</H1>");
		html.append("<H2>Results</H2>");
		html.append("<B>Type:</B>"
				+ Tournament.gameTypeToString(result.getTournamentGameType()));
		html.append("<BR><B>Date:</B>" + result.getTournamentDate());
		html
				.append("<BR><B>Total run time:</B>"
						+ ((double) result.getTotalTournamentTime() / 1000.0)
						+ " secs");
		html.append("<TABLE BORDER=\"1\">");
		html
				.append("<TR><TH>Ranking<TH>Player<TH>Strategy<TH>Games Played<TH>Won<TH>Tied<TH>Lost<TH>Total Points");
		int i = 0;
		for (Enumeration e = result.getPlayers().elements(); e
				.hasMoreElements();) {
			Player p = (Player) e.nextElement();
			Object[] args = { new Integer(++i), p.getName(),
					p.getStrategy().getFullName(), new Integer(p.gamesPlayed),
					new Integer(p.gamesWon), new Integer(p.gamesTied),
					new Integer(p.gamesLost), new Double(p.getResult()) };
			html
					.append(MessageFormat
							.format(
									"<TR><TD>{0}<TD>{1}<TD>{2}<TD>{3}<TD>{4}<TD>{5}<TD>{6}<TD>{7}",
									args));
		}
		html.append("</TABLE>");
		html.append(FOOTER);
		return html.toString();
	}

}
