/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.tools;

import ipdlx.Strategy;
import ipdlx.strategy.*;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.jar.Attributes;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

public class PDFileHandler extends URLClassLoader {

	public final static String STRATEGY_ENTRY_KEY = "Main-Strategy";

	public PDFileHandler(URL url) {
		this(new URL[] { url }, Strategy.class.getClassLoader());
	}

	public PDFileHandler(URL url, ClassLoader parent) {
		this(new URL[] { url }, parent);
	}

	public PDFileHandler(URL[] urls) {
		this(urls, Strategy.class.getClassLoader());
	}

	public PDFileHandler(URL[] urls, ClassLoader parent) {
		super(urls, parent);
	}

	public Strategy getStrategy(String className) {
		try {
			if (className.endsWith(".class")) {
				className = className.substring(0, className.indexOf(".class"));
//		ATF modification
//				ALLC strat1 = new ALLC();
//				return strat1;
			}
			/*
			try {
				URL loc = new URL("file:/Users/atf/Documents/cmc_class/ipdlx_src/");
				this.addURL(loc);
			} catch (Exception e) {
				System.out.println("Bad URL");
			}
			System.out.println("URLs:");
			for (URL u: this.getURLs()) {
				System.out.println(u.toString());
			}
			*/
			System.out.print("Loading class '" + className + "' ... ");
//    ATF modification -- fix exception
//			Class c = findClass(className);
			Class c = Class.forName("ipdlx.strategy." + className);
			System.out.print("Verifying '" + c.getName() + "' ... ");
			if (verify(c)) {
				System.out.println("ok!");
				return instantiateFromClass(c);
			} else {
				System.out.println("fault!");
			}
		} catch (ClassNotFoundException cnfe) {
			cnfe.printStackTrace();
			return null;
		} catch (NoClassDefFoundError ncde) {
			ncde.printStackTrace();
			return null;
		} catch (ClassFormatError cfe) {
			cfe.printStackTrace();
			return null;
		}
		return null;
	}

	public Strategy getStrategyFromJarFile(File jarFile, String strategyName) {
		try {
			return getStrategyFromJarFile(new JarFile(jarFile), strategyName);
		} catch (IOException ioe) {
			ioe.printStackTrace();
			return null;
		}
	}

	public Strategy getStrategyFromJarFile(JarFile jarFile, String strategyName) {
		Class c = getStrategyClassFromJarFile(jarFile, strategyName);
		if (c != null) {
			Strategy s = instantiateFromClass(c);
			return s;
		}
		return null;
	}

	public Class getStrategyClassFromJarFile(JarFile jarFile,
			String strategyName) {
		Class c = getStrategyClassFromManifest(jarFile);
		if (c == null) {
			System.out
					.println("Strategy jar '"
							+ jarFile.getName()
							+ "' failed to load from manifest.  Trying loading from class name '"
							+ strategyName + "' ...");
			c = getStrategyClassFromName(jarFile, strategyName);
		}

		if (c == null) {
			System.out.println("Strategy jar '" + jarFile.getName()
					+ "' failed to load from class name '" + strategyName
					+ "'.  Trying single entry ...");
			c = getStrategyClassSingleEntry(jarFile);
		}
		return c;
	}

	private Strategy instantiateFromClass(Class c) {
		try {
			Strategy strategy = (Strategy) c.newInstance();
			return strategy;
		} catch (InstantiationException ie) {
			ie.printStackTrace();
			return null;
		} catch (IllegalAccessException iae) {
			iae.printStackTrace();
			return null;
		}
	}

	public Class getStrategyClassSingleEntry(JarFile jarFile) {
		for (Enumeration e = jarFile.entries(); e.hasMoreElements();) {
			String entryName = ((JarEntry) e.nextElement()).getName();
			if (entryName.endsWith(".class")) {
				entryName = entryName.substring(0, entryName.indexOf(".class"));
				if (entryName.startsWith("classes/")) {
					entryName = entryName.substring(entryName
							.indexOf("classes/") + 8);
					/*
					 * try { addURL(new URL("classes/")); } catch (Exception
					 * exc) { exc.printStackTrace(); }
					 */
				}
				entryName = entryName.replace('/', '.');
				System.out.print("Loading class single entry '" + entryName
						+ "' ... ");
				try {
					Class c = findClass(entryName);
					System.out.print("Verifying '" + c.getName() + "' ... ");
					if (verify(c)) {
						System.out.println("ok!");
						return c;
					} else {
						System.out.println("fault!");
					}
				} catch (ClassNotFoundException cnfe) {
					// System.out.println("Not a strategy");
					cnfe.printStackTrace();
				} catch (NoClassDefFoundError ncdfe) {
					ncdfe.printStackTrace();
				}
			}
		}
		return null;
	}

	public boolean verify(Class c) {

		return Strategy.class.isAssignableFrom(c) && c != Strategy.class;
	}

	public static URL fileToJarURL(File file, String suffix)
			throws MalformedURLException {
		String url = "jar:" + file.toURL().toString() + "!/" + suffix;
		// System.out.println("CREATING= " + url);
		return new URL(url);
	}

	public Class getStrategyClassFromName(JarFile jarFile,
			String strategyClassName) {
		if (strategyClassName != null) {
			try {
				if (strategyClassName.endsWith(".class")) {
					strategyClassName = strategyClassName.substring(0,
							strategyClassName.indexOf(".class"));
				}

				if (strategyClassName.startsWith("classes/")) {
					strategyClassName = strategyClassName
							.substring(strategyClassName.indexOf("classes/") + 8);
				}
				System.out.print("Loading class '" + strategyClassName
						+ "' ... ");
				Class c = findClass(strategyClassName);
				System.out.print("Verifying '" + c.getName() + "' ... ");
				if (verify(c)) {
					System.out.println("ok!");
					return c;
				} else {
					System.out.println("ok!");
				}
			} catch (ClassNotFoundException cnfe) {
				cnfe.printStackTrace();
			} catch (NoClassDefFoundError ncdfe) {
				ncdfe.printStackTrace();
			}
		}
		return null;
	}

	public Class getStrategyClassFromManifest(JarFile jarFile) {
		try {
			Manifest manifest = jarFile.getManifest();
			if (manifest != null) {
				Attributes atts = manifest.getMainAttributes();
				String strategyClassName = atts.getValue(STRATEGY_ENTRY_KEY);
				Class c = getStrategyClassFromName(jarFile, strategyClassName);
				return c;
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}

	/*
	 * public boolean verify(Object obj) { if (obj instanceof Strategy) { return
	 * true; } return false; }
	 */

}