/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.tools;

import java.util.Random;

public abstract class RandomGenerator {

	private static java.util.Random random;

	private static long randomSeed;

	private static float deviation;

	static {
		randomSeed = System.currentTimeMillis();
		random = new Random(randomSeed);
	}

	public static double gaussianDouble() {
		return gaussianDouble(deviation);
	}

	public static void setDeviation(float dev) {
		RandomGenerator.deviation = dev;
	}

	/**
	 * Generates a random double from a Gaussian distribution with the specified
	 * deviation.
	 * <p>
	 * 
	 * @param dev
	 *            the desired deviation.
	 * @return a random double from a Gaussian distribution with deviation
	 *         <code>dev</code>.
	 */
	public static double gaussianDouble(float dev) {
		return random.nextGaussian() * dev;
	}

	public static double randomDouble(double lo, double hi) {
		return (hi - lo) * random.nextDouble() + lo;
	}

	public static double gaussianRandom(double mean, float deviation) {
		return mean + gaussianDouble(deviation);
	}

	public static final void main(String[] args) {
		javax.swing.JFrame f = new javax.swing.JFrame("Test");
		f.setSize(300, 300);
		f.setVisible(true);
		javax.swing.JPanel p = new javax.swing.JPanel() {
			public void paint(java.awt.Graphics g) {
				int size = 200;
				int[] bin = new int[size];
				RandomGenerator.setDeviation(0.05f);
				for (int i = 0; i < 10000; i++) {
					int index = (int) ((size / 2) + gaussianRandom(size, 0.05f));
					if (index >= 0 && index < size) {
						bin[index]++;
						g.drawLine(index, 0, index, bin[index]);
					}
				}
			}
		};
		f.getContentPane().add(p);

	}

}